# pylint: disable=W0703,W0612,W0621,W0614,C0301,C0103,R0914,R0912,W0702,W0603\
"""
#-------------------------------------------------------------------------------
# Name      :  es-feed.py
# Purpose   :  This script is used to process and feed visualizations to Elasticsearch
# Author    :  Rebaca
# Created   :  06-02-2017
# Copyright :  (c) Rebaca 2017
#-------------------------------------------------------------------------------
"""

import os, sys, json
import logging
from copy import deepcopy
from optparse import OptionParser
from datetime import datetime

import pprint

try:
    from elasticsearch import Elasticsearch
except:
    try:
        from pymodules.elasticsearch import Elasticsearch
    except:
        print "Module Error: elasticsearch module not found. Install elasticsearch(pip install elasticsearch)."

class EsFeeder(object):
    """
    ElasticSearch data feeder
    """
    
    def __init__(self, elastic_host, file_path):
        """  Initialize the EsFeeder object

        @args:
            elastic_host  : ElasticSearch host address
            file_path     : JSON file path
        """
        
        if not elastic_host: #error!
            logging.error("ElasticSearch host address not passed!")
            raise ValueError("ElasticSearch host address not passed!")
        if not file_path: #error!
            logging.error("JSON file path not passed!")
            raise ValueError("JSON file path not passed!")
        
        self.elastic_host = elastic_host
        self.file_path = file_path
        self.es = self._connect_es()

    def _connect_es(self):
        """
        Initializing the ElasticSearch connection
        """
        
        try:
            es = Elasticsearch([self.elastic_host])    
        except:
            logging.error("Could not connect to ElasticSearch host %s" %(self.elastic_host))
            raise
        return es
    
    def start_processing(self):
        files = []
        if (os.path.isdir(self.file_path)):
            files  = [os.path.join(self.file_path, file_name) for file_name in os.listdir(self.file_path) if (os.path.splitext(file_name)[1] in ['.json', '.JSON']) and (not os.path.isdir(os.path.join(self.file_path, file_name)))]
        else:
            files.append(self.file_path)
            
        for visual_file in files:    
            with open(visual_file) as visualizations:    
                data = json.load(visualizations)
                
                for visual in data:
                    try:
                        res = self.es.index(index=".kibana", doc_type=visual['_type'], id=visual['_id'], body=visual['_source'])
                        logging.info(res)
                        print res
                    except:
                        raise
                        logging.error("Could not post data to ElasticSearch host %s" %(self.elastic_host))
                        
def getOptsAndArgs(args):
    """
    Getting options and arguments passed to this script

    param args: in place system arguments sent to the script
    """

    parser = OptionParser()
    parser.add_option('-e', '--elastic',
                      action='store', dest='elastic_host',
                      help='ElasticSearch Host'
                      )
                      
    parser.add_option('-f', '--filepath',
                      action='store', dest='file_path',
                      help='JSON File Path'
                      )                    

    parser.add_option('-l', '--log',
                      action='store_true', dest='writeLog',
                      default=False, help='Console output to log file'
                      )
    
    options, args = parser.parse_args()

    return options, args


#__main__
if __name__ == '__main__':
    """
    Usage: python es-visual.py --elastic http://192.168.15.169:9200/ --filepath dashboard.json
    """
    
    options, args = getOptsAndArgs(sys.argv[1:])
    
    log_file = os.path.join(os.path.abspath(os.path.dirname(__file__)), "logs", "outputFile_%s.log" % \
                    datetime.now().strftime("%Y_%m_%d_%H_%M_%S"))
    if options.writeLog:
        logging.basicConfig(filename=log_file, format='%(asctime)s %(message)s', \
                            datefmt='%m/%d/%Y %I:%M:%S %p', level=logging.INFO)
        logging.info("Writing to log file : %s" % (log_file) )
        logging.info("Parameters passed to script : %s" % (options) )
        
    
    if not options.elastic_host or not options.file_path:
        logging.error("Invalid or partial arguments passed : %s" % (str(options)) )
        raise ValueError("Provide the following params - elastic host and file path")
    else:
        es_feeder = EsFeeder(options.elastic_host, options.file_path)
        logging.info("Data processing started for file %s and posting to ElasticSearch " % (options.file_path) )
        es_feeder.start_processing()
